{Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730}

procedure SGECO (  var A : MATRIX ;
                  lda, n : integer ;
                var IPvt : IVECTOR ;
               var RCond : real ;
                   var z : VECTOR ) ;
     {                                                     }
     { PROGRAM:                                            }
     {                                                     }
     {    SGECO  - Factors a real matrix by Gaussian       }
     {             elimination and computes the condition  }
     {             RCond of the matrix.                    }
     {                                                     }
     { VERSION:                  DATE:                     }
     {                                                     }
     {    1.0/TURBO Pascal 2.0      02/22/85               }
     {    FORTRAN                   08/14/78               }
     {                                                     }
     { DESCRIPTION:                                        }
     {                                                     }
     {                             1                       }
     {    RCond = ---------------------------------------- }
     {            norm(A) * (Estimate of norm(inverse(A))) }
     {                                                     }
     {    where;                                           }
     {                                                     }
     {       Estimate = norm(z)/norm(y)                    }
     {       A * z = y                                     }
     {       trans(A) * y = E                              }
     {                                                     }
     {    and trans(A) is transpose of A. Components of E  }
     {    are chosen to cause maximum local growth in the  }
     {    elements of w where;                             }
     {                                                     }
     {       trans(U) * w = E                              }
     {                                                     }
     {    Vectors are rescaled to avoid overflow.          }
     {                                                     }
     {    If RCond is not needed, SGEFA is slightly faster.}
     {                                                     }
     {    To compute:                                      }
     {                                                     }
     {       A * x = b       , follow SGECO by SGESL.      }
     {       inverse(A) * c  , follow SGECO by SGESL.      }
     {       determinant(A)  , follow SGECO by SGEDI.      }
     {       inverse(A)      , follow SGECO by SGEDI.      }
     {                                                     }
     {    On entry;                                        }
     {                                                     }
     {       A     - Matrix to be factored.                }
     {       lda   - Leading dimension of matrix  A.       }
     {       n     - Order of matrix  A.                   }
     {                                                     }
     {    On exit;                                         }
     {                                                     }
     {       A     - Upper triangular matrix and  multi-   }
     {               pliers used to obtain it. Factoriza-  }
     {               tion can be written;                  }
     {                  A = L * U                          }
     {               where L  is a product of permutation  }
     {               and unit lower triangular matrices    }
     {               and  U  is upper triangular.          }
     {                                                     }
     {       IPvt  - Pivot index vector.                   }
     {                                                     }
     {       RCond - Estimate of reciprocal condition of A.}
     {               For system  A * x = B , relative per- }
     {               turbations in  A  and  b  of size     }
     {               Epsilon  may cause relative perturba- }
     {               tions in  x  of size  Epsilon/RCond . }
     {               If  RCond  is so small that the logi- }
     {               cal expression;                       }
     {                  (1.0 + RCond) = 1.0                }
     {               is true, then  A  may be singular to  }
     {               workingprecision.  In particular,     }
     {               RCond  is zero  if exact singularity  }
     {               is detected or the estimate under-    }
     {               flows.                                }
     {                                                     }
     {       z     - Work vector whose contents are        }
     {               usually unimportant. If A is close to }
     {               a singular matrix, then  z  is an     }
     {               approximate null vector in the sense  }
     {               that;                                 }
     {                  norm(A*z) = RCond*norm(A)*norm(z)  }
     {                                                     }
     { SUBPROGRAMS:                                        }
     {                                                     }
     {    SGEFA      from LINPACK                          }
     {    SAXPY      from BLAS                             }
     {    SDOT       from BLAS                             }
     {    SSCAL      from BLAS                             }
     {    SASUM      from BLAS                             }
     {                                                     }
     { AUTHORS:                                            }
     {                                                     }
     {    Cleve Moler                                      }
     {    University of New Mexico                         }
     {    Argonne National Laboratories                    }
     {                                                     }
     {    PASCAL Translation;                              }
     {                                                     }
     {    Adam Fritz                                       }
     {    133 Main Street                                  }
     {    Afton, New York 13730                            }
     {                                                     }

var
     ek, t, wk, wkm : real ;
     anorm, s, sm, ynorm : real ;
     InfoCode       : integer ;
     j, k, kb, kp1, l : integer ;

begin
                                { Compute 1-norm of A }
   anorm := 0.0 ;
   for j := 1 to n do begin
      s := SASUM(n, A[1,j], lda) ;
      if anorm < s then
         anorm := s
   end ;
                                { Factor }
   SGEFA(A, lda, n, IPvt, InfoCode) ;
                                { Solve trans(U) * w = E }
   ek := 1.0 ;
   for j := 1 to n do
      z[j] := 0.0 ;
   for k := 1 to n do begin
      if z[k] > 0.0 then
         ek := -Abs(ek)
      else
         ek := Abs(ek) ;
      if Abs(ek-z[k]) > Abs(A[k,k]) then begin
         s := Abs(A[k,k])/Abs(ek-z[k]) ;
         SSCAL(n, s, z[1], 1) ;
         ek := s * ek
      end ;
      wk := ek - z[k] ;
      wkm := -ek - z[k] ;
      s := Abs(wk) ;
      sm := Abs(wkm) ;
      if A[k,k] <> 0.0 then begin
         wk := wk/A[k,k] ;
         wkm := wkm/A[k,k]
      end
      else begin
         wk := 1.0 ;
         wkm := 1.0
      end ;
      kp1 := k + 1 ;
      if kp1 <= n then begin
         for j := kp1 to n do begin
            sm := sm + Abs(z[j] + wkm * A[k,j]) ;
            z[j] := z[j] + wk * A[k,j] ;
            s := s + Abs(z[j])
         end ;
         if s < sm then begin
            t := wkm - wk ;
            wk := wkm ;
            for j := kp1 to n do
               z[j] := z[j] + t * A[k,j]
         end
      end ;
      z[k] := wk
   end ;
   s := 1.0/SASUM(n, z[1], 1) ;
   SSCAL(n, s, z[1], 1) ;
                                { Solve trans(L) * y = w }
   for kb := 1 to n do begin
      k := n + 1 - kb ;
      if k < n then
         z[k] := z[k] + SDOT(n-k, A[k+1,k], lda, z[k+1], 1) ;
      if Abs(z[k]) > 1.0 then begin
         s := 1.0/Abs(z[k]) ;
         SSCAL(n, s, z[1], 1)
      end ;
      l := IPvt[k] ;
      t := z[l] ;
      z[l] := z[k] ;
      z[k] := t
   end ;
   s := 1.0/SASUM(n, z[1], 1) ;
   SSCAL(n, s, z[1], 1) ;
                                { Solve L * v = y }
   ynorm := 1.0 ;
   for k := 1 to n do begin
      l := IPvt[k] ;
      t := z[l] ;
      z[l] := z[k] ;
      z[k] := t ;
      if k < n then
         SAXPY(n-k, t, A[k+1,k], lda, z[k+1], 1) ;
      if Abs(z[k]) > 1.0 then begin
         s := 1.0/Abs(z[k]) ;
         SSCAL(n, s, z[1], 1) ;
         ynorm := s * ynorm
      end
   end ;
   s := 1.0/SASUM(n, z[1], 1) ;
   SSCAL(n, s, z[1], 1) ;
   ynorm := s * ynorm ;
                                { Solve  U * z = v }
   for kb := 1 to n do begin
      k := n + 1 - kb ;
      if Abs(z[k]) > Abs(A[k,k]) then begin
         s := Abs(A[k,k])/Abs(z[k]) ;
         SSCAL(n, s, z[1], 1) ;
         ynorm := s * ynorm
      end ;
      if A[k,k] <> 0.0 then
         z[k] := z[k]/A[k,k]
      else
         z[k] := 1.0 ;
      t := -z[k] ;
      SAXPY(k-1, t, A[1,k], lda, z[1], 1)
   end ;
                                { Set znorm = 1.0 }
   s := 1.0/SASUM(n, z[1], 1) ;
   SSCAL(n, s, z[1], 1) ;
   ynorm := s * ynorm ;
   if anorm <> 0.0 then
      RCond := ynorm/anorm
   else
      RCond := 0.0
end ;

{Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730}
