{Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730}

procedure SGEFA (  var A : MATRIX ;
                  lda, n : integer ;
                var IPvt : IVECTOR ;
            var InfoCode : integer ) ;
     {                                                     }
     { PROGRAM:                                            }
     {                                                     }
     {    SGEFA - Factors Real Single Precision Matrix     }
     {            Using Gaussian Elimination.              }
     {                                                     }
     { VERSION:                 DATE:                      }
     {                                                     }
     {     1.0/TURBO Pascal 2.0    02/22/85                }
     {     FORTRAN                 08/14/78                }
     {                                                     }
     { DESCRIPTION:                                        }
     {                                                     }
     {     SGEFA is usually called by SGECO, but can be    }
     {     called directly with a saving in time if RCOND  }
     {     is not needed.                                  }
     {                                                     }
     {     (time for SGECO) = (1 + 9/n)*(time for SGEFA)   }
     {                                                     }
     {     On entry;                                       }
     {                                                     }
     {        A     - Matrix to be factored.               }
     {        lda   - Leading dimension of matrix A.       }
     {        n     - Order of matrix A.                   }
     {                                                     }
     {     On exit;                                        }
     {                                                     }
     {        A     - Upper triangular matrix and multi-   }
     {                pliers used to obtain it. Factoriza- }
     {                tion can be written;                 }
     {                   A = L * U                         }
     {                where L is a product of permutation  }
     {                and unit lower triangular matrices   }
     {                and U is upper triangular.           }
     {                                                     }
     {        IPvt  - Pivot index vector.                  }
     {                                                     }
     {        InfoCode -                              }
     {            = 0 Normal value.                        }
     {            = k If  u[k,k] = 0.0. This is not an     }
     {                error condition for this procedure,  }
     {                but indicates that SGESL or SGEDI    }
     {                will divide by zero if called.  Use  }
     {                RCOND from SGECO for reliable indi-  }
     {                cation of singularity.               }
     {                                                     }
     { SUBPROGRAMS:                                        }
     {                                                     }
     {    SAXPY       from BLAS                            }
     {    SSCAL       from BLAS                            }
     {    ISAMAX      from BLAS                            }
     {                                                     }
     { AUTHORS:                                            }
     {                                                     }
     {    Cleve Moler                                      }
     {    University of New Mexico                         }
     {    Argonne National Laboratories                    }
     {                                                     }
     {    PASCAL translation;                              }
     {                                                     }
     {    Adam Fritz                                       }
     {    133 Main Street                                  }
     {    Afton, New York 13730                            }
     {                                                     }

var
     j, k, l        : integer ;
     kp1, nm1       : integer ;
     t              : real ;

begin
                                { Gaussian Elimination with Partial Pivoting }
   InfoCode := 0 ;
   if n > 0 then begin
      nm1 := n - 1 ;
      for k := 1 to nm1 do begin
         kp1 := k + 1 ;
                                { Find l = Pivot Index }
         l := ISAMAX(n-k+1, A[k,k], lda) + k - 1 ;
         IPvt[k] := l ;
                                { Zero Pivot Implies Column Triangularized }
         if A[l,k] <> 0.0 then begin
                                { Interchange if Necessary }
            if l <> k then begin
               t := A[l,k] ;
               A[l,k] := A[k,k] ;
               A[k,k] := t
            end ;
                                { Compute Multipliers }
            t := -1.0/A[k,k] ;
            SSCAL(n-k, t, A[k+1,k], lda) ;
                                { Row Elimination with Column Indexing }
            for j := kp1 to n do begin
               t := A[l,j] ;
               if l <> k then begin
                  A[l,j] := A[k,j] ;
                  A[k,j] := t
               end ;
               SAXPY(n-k, t, A[k+1,k], lda, A[k+1,j], lda)
            end
         end
         else
            InfoCode := k
      end
   end ;
   IPvt[n] := n ;
   if A[n,n] = 0.0 then
      InfoCode := n
end ;

{Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730}
