{Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730}

procedure SGESL (  var A : MATRIX ;
                  lda, n : integer ;
                var IPvt : IVECTOR ;
                   var b : VECTOR ;
                 JobCode : integer ) ;
     {                                                     }
     { PROGRAM:                                            }
     {                                                     }
     {    SGESL - Solve General Real Linear System.        }
     {                                                     }
     { VERSION:                 DATE:                      }
     {                                                     }
     {     1.0/TURBO Pascal 2.0    02/22/85                }
     {     FORTRAN                 08/14/78                }
     {                                                     }
     { DESCRIPTION:                                        }
     {                                                     }
     {    Solves real single precision system;             }
     {                                                     }
     {       A * x = b  or  trans(A) * x = b               }
     {                                                     }
     {    using factors computed by SGECO or SGEFA.        }
     {                                                     }
     {    On entry;                                        }
     {                                                     }
     {       A      - Output from SGECO or SGEFA           }
     {       lda    - Leading dimension of matrix A        }
     {       n      - Order of matrix A.                   }
     {       IPvt   - Pivot index vector from SGECO or     }
     {                SGEFA.                               }
     {       b      - Right hand side vector               }
     {       JobCode -                                     }
     {             =0 solve  A * x = b ,                   }
     {             >0 solve  trans(A) * x = b  where       }
     {                trans(A) is the transpose.           }
     {                                                     }
     {    On exit;                                         }
     {                                                     }
     {       b      - Solution vector  x .                 }
     {                                                     }
     {    Error Condition;                                 }
     {                                                     }
     {       Division by zero will occur if the input      }
     {       factor contains a zero on the diagonal. This  }
     {       indicates singularity but can be caused by    }
     {       improper arguments or improper setting of lda.}
     {       It will occur if the procedures are called    }
     {       correctly and SGECO has set RCOND greater     }
     {       than 0.0 or SGEFA has set InfoCode            }
     {       different from 0.                             }
     {                                                     }
     {    To compute;                                      }
     {                                                     }
     {       inverse(A) * c                                }
     {                                                     }
     {    where  c  is a matrix with  p  columns;          }
     {                                                     }
     {       SGECO(A, lda, n, IPvt, RCOND, z) ;            }
     {       if RCOND ... ok then                          }
     {          for j := 1 to p do                         }
     {             SGESL(A, lda, n, IPvt, c[1,j], false) ; }
     {                                                     }
     { SUBPROGRAMS:                                        }
     {                                                     }
     {    SAXPY    from BLAS                               }
     {    SDOT     from BLAS                               }
     {                                                     }
     { AUTHORS:                                            }
     {                                                     }
     {    Cleve Moler                                      }
     {    University of New Mexico                         }
     {    Argonne National Laboratories                    }
     {                                                     }
     {    PASCAL translation;                              }
     {                                                     }
     {    Adam Fritz                                       }
     {    133 Main Street                                  }
     {    Afton, New York 13730                            }
     {                                                     }

var
     k, kb, l, nm1  : integer ;
     t              : real ;

begin
   nm1 := n - 1 ;
   if JobCode = 0 then begin
                                { Solve  A * x = b  }
      if nm1 > 0 then begin
         for k := 1 to nm1 do begin
            l := IPvt[k] ;
            t := b[l] ;
            if l <> k then begin
               b[l] := b[k] ;
               b[k] := t
            end ;
            SAXPY(n-k, t, A[k+1,k], lda, b[k+1], 1)
         end
      end ;
                                { Solve  U * x = y  }
      for kb := 1 to n do begin
         k := n + 1 - kb ;
         b[k] := b[k]/A[k,k] ;
         t := -b[k] ;
         SAXPY(k-1, t, A[1,k], lda, b[1], 1)
      end
   end
                                { Solve  trans(A) * x = b  }
   else begin
                                { Solve  trans(U) * y = b  }
      for k := 1 to n do begin
         t := SDOT(k-1, A[1,k], lda, b[1], 1) ;
         b[k] := (b[k] - t)/A[k,k]
      end ;
                                { Solve trans(L) * x = y  }
      if nm1 > 0 then begin
         for kb := 1 to nm1 do begin
            k := n - kb ;
            b[k] := b[k] + SDOT(n-k, A[k+1,k], lda, b[k+1], 1) ;
            l := IPvt[k] ;
            if l <> k then begin
               t := b[l] ;
               b[l] := b[k] ;
               b[k] := t
            end
         end
      end
   end
end ;

{Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730}
